package com.yaoyicloud.easier.common.file.oss;

import org.springframework.boot.context.properties.ConfigurationProperties;

import lombok.Data;

/**
 * aws 配置信息 配置文件添加： oss: enable: true endpoint: <a href="http://127.0.0.1:9000">...</a> #pathStyleAccess 采用nginx反向代理或者AWS S3
 * 配置成true，支持第三方云存储配置成false pathStyleAccess: false access-key: <access-key> secret-key: <secret-key> bucket-name: <bucket-name> region:
 * custom-domain:<a href="https://oss.xxx.com/">...</a><bucket-name>
 * <p>
 * bucket 设置公共读权限
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2022-12-02 21:23
 */
@Data
@ConfigurationProperties(prefix = "oss")
public class OssProperties {

    /**
     * 对象存储服务的URL
     */
    private String endpoint;

    /**
     * 自定义域名
     */
    private String customDomain;

    /**
     * true path-style nginx 反向代理和S3默认支持 pathStyle {<a href="http://endpoint/bucketname">...</a>} false supports virtual-hosted-style 阿里云等需要配置为
     * virtual-hosted-style 模式{<a href="http://bucketname.endpoint">...</a>}
     */
    private Boolean pathStyleAccess = true;

    /**
     * 应用ID
     */
    private String appId;

    /**
     * 区域
     */
    private String region;

    /**
     * Access key就像用户ID，可以唯一标识你的账户
     */
    private String accessKey;

    /**
     * Secret key是你账户的密码
     */
    private String secretKey;

    /**
     * 最大线程数，默认： 100
     */
    private Integer maxConnections = 100;

}
