package com.yaoyicloud.factory;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.util.Map;
import com.deepoove.poi.data.DocxRenderData;
import com.deepoove.poi.xwpf.NiceXWPFDocument;
import org.apache.poi.xwpf.usermodel.BodyElementType;
import org.apache.poi.xwpf.usermodel.IBodyElement;
import org.apache.poi.xwpf.usermodel.XWPFParagraph;
import org.apache.poi.xwpf.usermodel.XWPFTable;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import org.w3c.dom.Node;

/**
 * 抽象渲染器
 *
 */
public abstract class CSOAbstractRender extends BasicAbstractRender {

    /*
     * 子模版路径
     */
    private String subTemplatePath = "D:\\excel\\subtemplate.docx";

    public CSOAbstractRender(String cwd) {
        super(cwd);
    }

    /**
     * Docx 渲染
     *
     * 
     * @param inputStream 模板内容
     * @return 本地文件目录
     * @throws IOException
     */
    @SuppressWarnings("checkstyle:ParameterNumber")
    public String renderDocx(Map<String, Object> dataMap, InputStream inputStream,
        ConfigureBuilder builder, String relationId, String moduleType)
        throws IOException {
        String resultPath = super.renderDocx(dataMap, inputStream, builder, relationId, moduleType);
        Configure config = builder.build();
        // 处理嵌套模板逻辑
        if (moduleType.equals("PromotionDetails")) {
            XWPFTemplate subTemplate = XWPFTemplate.compile(subTemplatePath, config).render(dataMap);
            ByteArrayOutputStream nestOutput = new ByteArrayOutputStream();
            subTemplate.write(nestOutput);
            ByteArrayInputStream nestInput = new ByteArrayInputStream(nestOutput.toByteArray());
            DocxRenderData docxRenderData = new DocxRenderData(nestInput);
            dataMap.put("nestfile", docxRenderData);
        }

        NiceXWPFDocument xwpfDocument = new NiceXWPFDocument(new FileInputStream(new File(resultPath)));
        List<XWPFTable> allTables = xwpfDocument.getAllTables();

        int toDeleteLine = 0;
        // 删除多余的表格
        for (int i = 0; i < allTables.size(); i++) {
            XWPFTable xwpfTable = allTables.get(i);
            if (allTables.get(i).getRows().size() <= 1) {
                Node domNode = xwpfTable.getCTTbl().getDomNode();
                Node parentNode = domNode.getParentNode();
                parentNode.removeChild(domNode);
                toDeleteLine++;
            }
        }

        // 删除空标题行
        for (int i = xwpfDocument.getXWPFDocument().getBodyElements().size() - 1; i >= 0; i--) {
            IBodyElement ele = xwpfDocument.getXWPFDocument().getBodyElements().get(i);
            if (ele.getElementType().equals(BodyElementType.PARAGRAPH)) {
                XWPFParagraph paragraph = (XWPFParagraph) ele;
                if (paragraph.getText().contains("EMPTY")) {
                    xwpfDocument.removeBodyElement(i);
                    if (i - 1 < 0) {
                        continue;
                    }
                    IBodyElement above = xwpfDocument.getXWPFDocument().getBodyElements().get(i - 1);
                    if (!ele.getElementType().equals(BodyElementType.PARAGRAPH)) {
                        continue;
                    }
                    XWPFParagraph paragraphAbove = (XWPFParagraph) above;
                    if (paragraphAbove.getText().equals("")) {
                        xwpfDocument.removeBodyElement(i - 1);
                        i--;
                    }
                }
            }
        }
        // ArrayList<String> templateDele = new ArrayList<>();
        // for (Map.Entry<String, Object> entry : specialMap.entrySet()) {
        // String key = entry.getKey();
        // Object value = entry.getValue();
        // if (value.equals("")) {
        // templateDele.add(key);
        // }
        // }
        // Map<String, String> keywordMap = new HashMap<>();
        // keywordMap.put("suggestion", "建议");
        // keywordMap.put("score", "分");
        // keywordMap.put("riskSummary", "-");
        // XWPFDocument doc = template.getXWPFDocument();
        // List<XWPFTable> tables = doc.getTables();
        // XWPFTable xwpfTable = tables.get(tables.size() - 1);
        // for (int i = xwpfTable.getNumberOfRows() - 1; i >= 0; i--) {
        // XWPFTableRow row = xwpfTable.getRow(i);
        // StringBuilder rowText = new StringBuilder();
        // for (XWPFTableCell cell : row.getTableCells()) {
        // rowText.append(cell.getText());
        // }
        // for (String codeKeyword : templateDele) {
        // String templateKeyword = keywordMap.get(codeKeyword);
        // if (rowText.toString().contains(templateKeyword)) {
        // xwpfTable.removeRow(i);
        // break;
        // }
        // }
        // }

        return this.docxResultPath;
    }

}
