package com.yaoyicloud.render.cso;

import java.io.IOException;
import java.io.InputStream;
import java.time.LocalDateTime;
import java.util.Map;
import cn.hutool.core.date.DateUtil;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.protobuf.util.JsonFormat;
import com.yaoyicloud.config.CommonDataCache;
import com.yaoyicloud.config.FilerepoProperties;
import com.yaoyicloud.factory.CSOAbstractRender;
import com.yaoyicloud.message.CSOProtos;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.MapUtils;

@Slf4j
public final class EntHeaderSectionRender extends CSOAbstractRender {
    private final FilerepoProperties filerepoProperties;
    private final CommonDataCache commonDataCache;

    public EntHeaderSectionRender(String cwd, FilerepoProperties filerepoProperties, CommonDataCache commonDataCache) {
        super(cwd);
        this.filerepoProperties = filerepoProperties;
        this.commonDataCache = commonDataCache;
    }

    @Override
    protected String getBasicPath() throws IOException {
        return filerepoProperties.getBasePath();
    }

    @Override
    protected String getReportImagePath() {
        return filerepoProperties.getReportImagePath();
    }

    /**
     *  Docx 渲染
     * @param info 数据
     * @param inputStream 模板内容
     * @param relationId  用于查询缓存
     * @return
     * @throws IOException
     */
    public String renderDocx(String info, InputStream inputStream, String relationId) throws IOException {
        log.info("开始渲染CSO企业报告封面模块，relationId: {}", relationId);

        // 配置POI-TL渲染器
        ConfigureBuilder builder = Configure.builder();
        builder.bind("drugDetails", this.indicatorsRenderPolicyToProtobuf());
        CSOProtos.ReportHeader.Builder header = CSOProtos.ReportHeader.newBuilder();

        JsonFormat.parser().merge(info, header);
        CSOProtos.ReportHeader defaultInstance = CSOProtos.ReportHeader.getDefaultInstance();
        CSOProtos.ReportHeader mergedProto = defaultInstance.toBuilder()
            .mergeFrom(header.build())
            .build();

        String completeJson = JsonFormat.printer()
            .includingDefaultValueFields()
            .print(mergedProto);

        ObjectMapper objectMapper = new ObjectMapper();
        Map<String, Object> data = objectMapper.readValue(completeJson, new TypeReference<Map<String, Object>>() {});
        commonDataCache.addData(relationId, "reportCreateTime", DateUtil.format(LocalDateTime.now(), "yyyy年MM月dd日 HH:mm:ss"));
        Map<String, Object> commonDataCacheData = commonDataCache.getData(relationId);
        if (MapUtils.isNotEmpty(commonDataCacheData)) {
            data.putAll(commonDataCacheData);
        }
        commonDataCache.addData(relationId, "totalPackageScore", data.get("totalPackageScore"));
        commonDataCache.addData(relationId, "totalApprovedScore", data.get("totalApprovedScore"));
        commonDataCache.addData(relationId, "taskUserNames", data.get("taskUserNames"));
        commonDataCache.addData(relationId, "submitedTaskScore", data.get("submitedTaskScore"));
        commonDataCache.addData(relationId, "createTime", data.get("createTime"));
        commonDataCache.addData(relationId, "serviceEndDate", data.get("serviceEndDate"));
        commonDataCache.addData(relationId, "sensitiveFlag", data.get("sensitiveFlag"));
        commonDataCache.addData(relationId, "scorePackageName", data.get("scorePackageName"));
        commonDataCache.addData(relationId, "entrustingCompany", data.get("entrustingCompany"));
        commonDataCache.addData(relationId, "taskFinishTime", data.get("taskFinishTime"));
        commonDataCache.addData(relationId, "serviceCompany", data.get("serviceCompany"));

        fillDefaultValues(data);
        try {
            // 渲染文档
            String resultPath = this.renderDocx(data, inputStream, builder, relationId, "Header");
            log.info("渲染CSO企业报告封面模块成功，文件路径: {}", resultPath);
            return resultPath;
        } catch (Exception e) {
            log.error("渲染CSO企业报告封面模块失败，relationId: {}", relationId, e);
            throw new IOException("文档渲染失败", e);
        }
    }

    /**
     * 填充默认值，确保所有必要字段都存在
     */
    private void fillDefaultValues(Map<String, Object> data)  {
        processJsonData(data, "drugDetails");

    }

}
