package com.yaoyicloud.render.platform.update;

import java.io.IOException;
import java.io.InputStream;
import java.util.Map;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.deepoove.poi.policy.RenderPolicy;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.protobuf.util.JsonFormat;
import com.yaoyicloud.config.CommonDataCache;
import com.yaoyicloud.config.FilerepoProperties;
import com.yaoyicloud.factory.BasicAbstractRender;
import com.yaoyicloud.message.FxyProtos;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.MapUtils;

@Slf4j
public final class AntiBriberyNewRender extends BasicAbstractRender {
    private final FilerepoProperties filerepoProperties;
    private final CommonDataCache commonDataCache;

    // 注入父类所需的 cwd 参数
    public AntiBriberyNewRender(String cwd, FilerepoProperties filerepoProperties, CommonDataCache commonDataCache) {
        super(cwd);
        this.filerepoProperties = filerepoProperties;
        this.commonDataCache = commonDataCache;
    }

    @Override
    protected String getBasicPath() throws IOException {
        return filerepoProperties.getBasePath();
    }

    @Override
    protected String getReportImagePath() {
        return filerepoProperties.getReportImagePath();
    }

    /**
     * Docx 渲染
     *
     * @param info 数据
     * @param inputStream 模板内容
     * @param relationId 关联ID
     * @return 本地文件目录
     * @throws IOException
     */
    public String renderDocx(String info, InputStream inputStream, String relationId, Integer counter)
        throws IOException {
        log.info("开始渲染其他风险报告模块，relationId: {}", relationId);

        // 配置POI-TL渲染器
        ConfigureBuilder builder = Configure.builder();
        RenderPolicy indicatorsRenderPolicy = this.indicatorsRenderPolicyToProtobuf();
        builder.bind("questionnaireItems", indicatorsRenderPolicy);
        builder.bind("otherRiskChecks", indicatorsRenderPolicy);

        builder.useSpringEL();
        // 解析数据

        // 通过默认protobuf实例来填充不存在的key
        FxyProtos.AntiBribery.Builder basicInfoBuilder = FxyProtos.AntiBribery.newBuilder();
        JsonFormat.parser().merge(info, basicInfoBuilder);

        FxyProtos.AntiBribery defaultInstance = FxyProtos.AntiBribery.getDefaultInstance();
        FxyProtos.AntiBribery mergedProto = defaultInstance.toBuilder()
            .mergeFrom(basicInfoBuilder.build())
            .build();

        String completeJson = JsonFormat.printer()
            .includingDefaultValueFields()
            .print(mergedProto);
        ObjectMapper objectMapper = new ObjectMapper();
        Map<String, Object> data = objectMapper.readValue(completeJson, new TypeReference<Map<String, Object>>() {});
        Map<String, Object> commonDataCacheData = commonDataCache.getData(relationId);
        if (MapUtils.isNotEmpty(commonDataCacheData)) {
            data.putAll(commonDataCacheData);
        }
        fillDefaultValues(data);
        data.put("levelInteger", counter);
        // Map<String, Object> antiBriberySummary = (Map<String, Object>) data.get("antiBriberySummary");
        // String riskSummary = (String) antiBriberySummary.get("riskSummary");
        // antiBriberySummary.put("riskSummary", riskSummary == null || riskSummary.isEmpty() ? "-" : riskSummary);

        try {
            // 渲染文档
            String resultPath = this.renderDocx(data, inputStream, builder, relationId, "antiBribery");
            log.info("其他风险报告模块渲染成功，文件路径: {}", resultPath);
            return resultPath;
        } catch (Exception e) {
            log.error("其他风险报告模块渲染失败，relationId: {}", relationId, e);
            throw new IOException("文档渲染失败", e);
        }
    }

    /**
     * 填充默认值，确保所有必要字段都存在
     */
    private void fillDefaultValues(Map<String, Object> data) {
        Map<String, Object> antiBriberySummary = (Map<String, Object>) data.get("antiBriberySummary");
        antiBriberySummary.replaceAll((k, v) -> v.equals("") ? "-" : v);

    }

}
