package com.yaoyicloud.render.platform.update;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.deepoove.poi.policy.RenderPolicy;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.protobuf.Descriptors;
import com.google.protobuf.util.JsonFormat;
import com.yaoyicloud.config.CommonDataCache;
import com.yaoyicloud.config.FilerepoProperties;
import com.yaoyicloud.factory.BasicAbstractRender;
import com.yaoyicloud.message.FxyProtos;


import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.MapUtils;

/**
 * BasicInfo渲染器
 *
 */
@Slf4j
public final class BasicInfoNewRender extends BasicAbstractRender {
    private final FilerepoProperties filerepoProperties;
    private final CommonDataCache commonDataCache;

    public BasicInfoNewRender(String cwd, FilerepoProperties filerepoProperties, CommonDataCache commonDataCache) {
        super(cwd);
        this.filerepoProperties = filerepoProperties;
        this.commonDataCache = commonDataCache;
    }

    @Override
    protected String getBasicPath() throws IOException {
        return filerepoProperties.getBasePath();
    }

    @Override
    protected String getReportImagePath() {
        return filerepoProperties.getReportImagePath();
    }

    /**
     * Docx 渲染
     *
     * @param info 数据
     * @param inputStream 模板内容
     * @return 本地文件目录
     * @throws IOException
     */
    public String renderDocx(String info, InputStream inputStream,
                             String relationId, Integer counter) throws IOException {
        log.info("开始渲染工商信息报告模块，relationId: {}", relationId);

        // 配置POI-TL渲染器
        ConfigureBuilder builder = Configure.builder();
        RenderPolicy indicatorsRenderPolicy = this.indicatorsRenderPolicy();
        RenderPolicy deleteLoopRowTableRenderPolicy = this.deleteLoopRowTableRenderPolicy();
        builder.bind("basicInfoChecks", indicatorsRenderPolicy);
        builder.bind("sameaddress", deleteLoopRowTableRenderPolicy);
        builder.bind("samecontactNumberDatas", deleteLoopRowTableRenderPolicy);
        builder.addPlugin('^', this.pictureRenderPolicy());
         builder.useSpringEL();
        // 通过默认protobuf实例来填充不存在的key
        FxyProtos.BasicInfo.Builder basicInfoBuilder = FxyProtos.BasicInfo.newBuilder();
        JsonFormat.parser().merge(info, basicInfoBuilder);

        FxyProtos.BasicInfo defaultInstance = FxyProtos.BasicInfo.getDefaultInstance();
        FxyProtos.BasicInfo mergedProto = defaultInstance.toBuilder()
            .mergeFrom(basicInfoBuilder.build())
            .build();

        String completeJson = JsonFormat.printer()
            .includingDefaultValueFields()
            .print(mergedProto);

        ObjectMapper objectMapper = new ObjectMapper();
        Map<String, Object> data = objectMapper.readValue(completeJson, new TypeReference<Map<String, Object>>() {});
        data.replaceAll((k, v) -> v.equals("") ? "-" : v);
        Map<String, Object> newAddtionMap = new HashMap<>();
        Descriptors.Descriptor descriptor = mergedProto.getDescriptorForType();
        for (Descriptors.FieldDescriptor field : descriptor.getFields()) {
            if (field.isRepeated()) {
                List<?> li = (List<?>) mergedProto.getField(field);
                if (li.size() == 0) {
                    newAddtionMap.put(field.getName() + "AltText", "未上传");
                }
            }
        }

        if (newAddtionMap.size() > 0) {
            data.putAll(newAddtionMap);
        }

        Map<String, Object> commonDataCacheData = commonDataCache.getData(relationId);
        if (MapUtils.isNotEmpty(commonDataCacheData)) {
            data.putAll(commonDataCacheData);
        }
        data.put("levelInteger", counter);
        fillBasicDefaultValues(data);
//        Map<String, Object> basicInfoSummary = (Map<String, Object>) data.get("basicInfoSummary");
//        basicInfoSummary.put("riskSummary",
//            basicInfoSummary.get("riskSummary").equals("") ? "-" : basicInfoSummary.get("riskSummary"));
        try {
            // 渲染文档
            String resultPath = this.renderDocx(data, inputStream, builder, relationId, "basicInfo");
            log.info("渲染工商信息报告模块成功，文件路径: {}", resultPath);
            return resultPath;
        } catch (Exception e) {
            log.error("渲染工商信息报告模块失败，relationId: {}", relationId, e);
            throw new IOException("文档渲染失败", e);
        }
    }

    /**
     * 填充默认值，确保所有必要字段都存在
     */
    private void fillBasicDefaultValues(Map<String, Object> data) {
        Map<String, Object> basicInfoSummary = (Map<String, Object>) data.get("basicInfoSummary");
        basicInfoSummary.replaceAll((k, v) -> v.equals("") ? "-" : v);
        Map<String, Object> platformExt = (Map<String, Object>) data.get("platformExt");
        platformExt.replaceAll((k, v) -> v.equals("") ? "-" : v);
    }

}
