package com.yaoyicloud.render.platform.update;

import java.io.IOException;
import java.io.InputStream;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.deepoove.poi.policy.RenderPolicy;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yaoyicloud.config.CommonDataCache;
import com.yaoyicloud.config.FilerepoProperties;

import com.yaoyicloud.factory.BasicAbstractRender;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.collections4.MapUtils;

@Slf4j
public final class PublicRecordNewRender extends BasicAbstractRender {
    private final FilerepoProperties filerepoProperties;
    private final CommonDataCache commonDataCache;

    public PublicRecordNewRender(String cwd, FilerepoProperties filerepoProperties, CommonDataCache commonDataCache) {
        super(cwd);
        this.filerepoProperties = filerepoProperties;
        this.commonDataCache = commonDataCache;
    }

    @Override
    protected String getBasicPath() throws IOException {
        return filerepoProperties.getBasePath();
    }

    @Override
    protected String getReportImagePath() {
        return filerepoProperties.getReportImagePath();
    }

    /**
     * Docx 渲染
     *
     * @param info 数据
     * @param inputStream 模板内容
     * @return 本地文件目录
     * @throws IOException
     */

    public String renderDocx(String info, InputStream inputStream,
        String relationId, Integer counter) throws IOException {
        log.info("开始渲染司法风险模块，relationId: {}", relationId);

        // 配置POI-TL渲染器
        ConfigureBuilder builder = Configure.builder();
        RenderPolicy indicatorsRenderPolicy = this.indicatorsRenderPolicyToProtobuf();

        builder.bind("dishonestPersons", indicatorsRenderPolicy);
        builder.bind("judicialRiskChecks", indicatorsRenderPolicy);

        builder.bind("severeViolations", indicatorsRenderPolicy);
        builder.useSpringEL();

        ObjectMapper objectMapper = new ObjectMapper();
        Map<String, Object> data = objectMapper.readValue(info, new TypeReference<Map<String, Object>>() {});
        data.replaceAll((k, v) -> v.equals("") ? "-" : v);

        fillBasicDefaultValues(data);

        Map<String, Object> publicRecordSummary = (Map<String, Object>) data.get("publicRecordSummary");
        publicRecordSummary.putIfAbsent("riskSummary", "-");
        publicRecordSummary.putIfAbsent("suggestion", "-");
        Map<String, Object> commonDataCacheData = commonDataCache.getData(relationId);
        if (MapUtils.isNotEmpty(commonDataCacheData)) {
            data.putAll(commonDataCacheData);
        }
        data.put("levelInteger", counter);
        try {
            // 渲染文档
            String resultPath = this.renderDocx(data, inputStream, builder, relationId, "publicRecord");
            log.info("渲染司法风险模块成功，文件路径: {}", resultPath);
            return resultPath;
        } catch (Exception e) {
            log.error("渲染司法风险模块失败，relationId: {}", relationId, e);
            throw new IOException("文档渲染失败", e);
        }
    }

    /**
     * 填充默认值，确保所有必要字段都存在
     */
    private void fillBasicDefaultValues(Map<String, Object> data) {

        List<Map<String, Object>> dishonestPersons = (List<Map<String, Object>>) data.get("dishonestPersons");
        if (CollectionUtils.isEmpty(dishonestPersons)) {
            Map<String, Object> defaultDishonestPerson = new HashMap<>();
            defaultDishonestPerson.put("id", "1");
            List<Map<String, Object>> defaultDishonestPersonList = Collections.singletonList(defaultDishonestPerson);
            data.put("dishonestPersons", defaultDishonestPersonList);

        }

        List<Map<String, Object>> severeViolations = (List<Map<String, Object>>) data.get("severeViolations");
        if (CollectionUtils.isEmpty(severeViolations)) {
            Map<String, Object> defaultSevereViolation = new HashMap<>();
            defaultSevereViolation.put("id", "1");
            List<Map<String, Object>> defaultSevereViolationList = Collections.singletonList(defaultSevereViolation);
            data.put("severeViolations", defaultSevereViolationList);
        }
    }
}
